/*
 * @(#)jazzdlg.java	1.00 20th January 2000
 * 
 * Modification Log:
 *
 */

package jazzmsz;

import java.awt.*;
import java.awt.event.*;


/**
 * A dialog box control with in-build icons.
 * Displays a dialog box with different icons and colors depending
 * on the message type. Has customizable buttons.
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 20th January, 2000
 */


public class jazzdlg extends Dialog implements ActionListener
{
    private GridBagLayout       gridbag             = null;
    private GridBagConstraints  gbConstr            = null;
    private Panel               ButtonPanel         = null;
    private jazzicon            icon                = null;
    private TextArea            txtMsg              = null;

    String                      sResp               = "";
    boolean                     bSelfDestroy        = true;
    boolean                     bFocusSet           = false;
    Component                   cFocusOn            = null;
    Frame                       ParentFrame         = null;
    static boolean              bForceSetFocus;

    public static final int     NONE = jazzicon.NONE;
    public static final int     INFORMATION = jazzicon.INFORMATION;
    public static final int     CONFIRM = jazzicon.CONFIRM;
    public static final int     HELP = jazzicon.HELP;
    public static final int     WARNING = jazzicon.WARNING;
    public static final int     EXCEPTION = jazzicon.EXCEPTION;
    public static final int     ERROR = jazzicon.ERROR;
    
    private static Color        bgCol, fgCol;
    private static Font         fnt;
    
    /**
     * Constructor
     *
     * @param   parent    parent frame, from where it is being called.
     * @param   sTitle    Title of the dialog box. Default title <b>Message</b> is used if it is null.
     * @param   bModal    whether it is modal.
     */
    public jazzdlg(Frame parent, String sTitle, boolean bModal)
    {
        super(parent, (null==sTitle) ? "Message" : sTitle, bModal);
        ParentFrame=parent;
        setJazzLayout();
    }

    /**
     * Constructor
     *
     * @param   parent    parent frame, from where it is being called.
     * @param   bModal    whether it is modal.
     */
    public jazzdlg(Frame parent, boolean bModal)
    {
        super(parent, bModal);
        ParentFrame=parent;
        setJazzLayout();
    }


    public static void setDlgGui(Color bgColIn, Color fgColIn, Font fntIn)
    {
        jazzdlg.bgCol = bgColIn;
        jazzdlg.fgCol = fgColIn;
        jazzdlg.fnt = fntIn;
    }
    
    /**
     * Shows a message box with the specified properties. 
     * Returns the name of the button pressed if it is modal.
     *
     * @param   sTitle       title of the dialog box.
     * @param   sMsg         message to be displayed.
     * @param   sButtons     array of names of buttons to be shown.
     * @param   iType        type of dialog: NONE, INFORMATION, WARNING, EXCEPTION, ERROR,...
     * @param   bWait        true if dialog is to be modal.
     * @param   calledFrom   container from where this is being called (used to center the dialog).
     *
     * @return     the name of the button pressed if the dialog is modal, empty string otherwise.
     */
    public static String showMsgBox (String sTitle, String sMsg, String [] sButtons, int iType, boolean bWait, Container calledFrom)
    {
        String      sResp = "";
        jazzdlg     msgbox;
        Frame       parentFrame;

        parentFrame = findFrame(calledFrom);
        msgbox = new jazzdlg(parentFrame, sTitle, bWait);
        msgbox.txtMsg.setText(sMsg);
        msgbox.icon.setIconCharacterType(iType);

        try
        {
            for (int iIndex=0; ; iIndex++)
            {
                Button b = new Button(sButtons[iIndex]);
                b.addActionListener(msgbox);
                msgbox.ButtonPanel.add(b);
                if(null == msgbox.cFocusOn) msgbox.cFocusOn = b;
            }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            // array out of bounds exception. ignore
        }

        msgbox.bSelfDestroy = !bWait;

        if (bWait)
        {
            calledFrom.setEnabled(false);   // parent should not get any events
        }

        msgbox.setVisible(true);
        if (bWait)
        {
            while (msgbox.sResp.equals(""))
            {
                try
                {
                    Thread.yield();
                    Thread.sleep(500);
                }
                catch(InterruptedException ie)
                {
                    // ie
                }
            }
            sResp = msgbox.sResp;
        }

        if (bWait)
        {
            calledFrom.setEnabled(true);
        }

        msgbox = null;
        return sResp;
    }
    
    /**
     * Sets the layout for the message dialog frame. Uses grid bag layout.
     */
    private void setJazzLayout()
    {
        icon        = new jazzicon();
        txtMsg      = new TextArea("", 5, 30, TextArea.SCROLLBARS_NONE);
        ButtonPanel = new Panel();
        gridbag     = new GridBagLayout();
        gbConstr    = new GridBagConstraints();
        
        txtMsg.setEditable(false);
        txtMsg.setEnabled(false);
        icon.setFont(new Font("Courier", Font.BOLD, 30));
        
        if(null != jazzdlg.bgCol) setBackground(jazzdlg.bgCol);
        if(null != jazzdlg.fgCol) setForeground(jazzdlg.fgCol);
        if(null != jazzdlg.fnt)   setFont(jazzdlg.fnt);
        setLayout(gridbag);
        
        gbConstr.ipadx = 0;
        gbConstr.ipady = 0;
        gbConstr.fill = GridBagConstraints.BOTH;
        gbConstr.anchor = GridBagConstraints.NORTHWEST;
        gbConstr.gridheight = GridBagConstraints.REMAINDER;
        gbConstr.gridwidth = GridBagConstraints.RELATIVE;
        gbConstr.weightx = 0;
        gbConstr.weighty = 0;
        gbConstr.gridx = 0;
        gbConstr.gridy = 0;
        gridbag.setConstraints(icon, gbConstr);
        add(icon);
        
        gbConstr.fill = GridBagConstraints.BOTH;
        gbConstr.gridheight = GridBagConstraints.RELATIVE;
        gbConstr.gridwidth = GridBagConstraints.REMAINDER;
        gbConstr.weightx = 1;
        gbConstr.weighty = 1;
        gbConstr.gridx = GridBagConstraints.RELATIVE;
        gbConstr.gridy = 0;
        gridbag.setConstraints(txtMsg, gbConstr);
        add(txtMsg);
        
        gbConstr.fill = GridBagConstraints.NONE;
        gbConstr.anchor = GridBagConstraints.CENTER;
        gbConstr.gridwidth = GridBagConstraints.REMAINDER;
        gbConstr.weightx = 1;
        gbConstr.weighty = 0;
        gbConstr.gridx = GridBagConstraints.RELATIVE;
        gbConstr.gridy = GridBagConstraints.RELATIVE;
        gridbag.setConstraints(ButtonPanel, gbConstr);
        ButtonPanel.setLayout(new GridLayout(1, 0, 5, 25));
        add(ButtonPanel);
    }

    /**
     * Lays out the screen, moves the screen to the center of the parent window, 
     * before making the dialog visible.
     */
    public void setVisible(boolean bShow)
    {
        if (true == bShow)
        {
            pack();

            Dimension d = getSize();
            d.width += 100;
            setSize(d);

            try
            {
                if (null != ParentFrame)
                {
                    Rectangle bounds = ParentFrame.getBounds();
                    Rectangle abounds = getBounds();

                    setLocation(bounds.x + (bounds.width - abounds.width) / 2,
                         bounds.y + (bounds.height - abounds.height) / 2);
                }
            }
            catch (Exception e)
            {
                // nothing
            }

            if (bForceSetFocus && (null != cFocusOn))
            {
                cFocusOn.requestFocus();
            }

            super.setVisible(true);
        }
        else
        {
            super.setVisible(false);
        }
    }

    /**
     * Hides the dialog if any button is pressed.
     */
    public void actionPerformed(ActionEvent event)
    {
        sResp = event.getActionCommand();
        setVisible(false);
        dispose();
    }
    
    /**
     * Finds the parent frame of the specified component.
     *
     * @param   calledFrom  container whoose parent to find. <br>
     *                      In this context, the container from where 
     *                      showMsgBox was called.
     * @return              the parent frame if found, null otherwise.
     */
    private static Frame findFrame(Container calledFrom)
    {
        if (null != calledFrom)
        {
            Container frameContainer;

            frameContainer = calledFrom;
            do
            {
                if (frameContainer instanceof Frame)
                {
                    return (Frame)frameContainer;
                }
            } while (null != (frameContainer = frameContainer.getParent()));
        }
        return null;
    }
}
