/*
 * @(#)jazzicon.java	1.00 20th January 2000
 * 
 * Modification Log:
 *
 */

package jazzmsz;

import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;


/**
 * A icon control with in-build icons.
 * Displays different icons depending on the message type. 
 * Has customizable buttons.
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 20th January, 2000
 */

public class jazzicon extends Canvas
{
    private Image       image;
    private Character   charIcon;
    private String      sIcon;

    public static final int     NONE = 0;
    public static final int     INFORMATION = 1;
    public static final int     CONFIRM = 2;
    public static final int     HELP = 3;
    public static final int     WARNING = 4;
    public static final int     EXCEPTION = 5;
    public static final int     ERROR = 6;
    
    public boolean setIconImage(String sResource)
    {
        if (null != (image = getImageFromResource(sResource)))
        {
            sIcon = null;
            charIcon = null;
            return true;
        }
        return false;
    }
    
    public void setIconCharacter(Character charIn)
    {
        if(null != (charIcon = charIn))
        {
            sIcon = charIcon.toString();
            image = null;
        }
    }
    
    public void setIconCharacterType(int iType)
    {
        switch(iType)
        {
            case INFORMATION:
                setIconCharacter(new Character('i'));
                setForeground(Color.blue);
                setBackground(Color.white);
                break;
            case HELP:
                setIconCharacter(new Character('h'));
                setForeground(Color.blue);
                setBackground(Color.white);
                break;
            case CONFIRM:
                setIconCharacter(new Character('?'));
                setForeground(Color.blue);
                setBackground(Color.white);
                break;
            case WARNING:
                setIconCharacter(new Character('!'));
                setForeground(Color.black);
                setBackground(Color.yellow);
                break;
            case EXCEPTION:
                setIconCharacter(new Character('e'));
                setForeground(Color.black);
                setBackground(Color.orange);
                break;
            case ERROR:
                setIconCharacter(new Character('E'));
                setForeground(Color.white);
                setBackground(Color.red);
                break;
            default:
                setIconCharacter(new Character(' '));
                Component c = getParent();
                if(null != c)
                {
                    setForeground(c.getForeground());
                    setBackground(c.getBackground());
                }
                break;
        }
        repaint();
    }
    
    /**
     * Paints the image on the canvas. Draws a border around it if has focus.
     */
    public void paint(Graphics g)
    {
        Dimension dim = getSize();

        if (null != image)
        {
            g.drawImage (image, 1, 1, dim.width-2, dim.height-2, this);
        }
        else
        {
            g.setColor(getBackground());
            g.fill3DRect(-1, -1, dim.width+2, dim.height+2, true);
            if(null != sIcon)
            {
                FontMetrics fm;
                g.setColor(getForeground());
                int iX, iY;
                iX = dim.width/2;
                iY = dim.height/2;
                fm = g.getFontMetrics();
                iX -= fm.stringWidth(sIcon)/2;
                iY += fm.getDescent();
                g.drawString(sIcon, iX, iY);
                if(!Character.isWhitespace(sIcon.charAt(0))) g.drawRect(0, 0, dim.width, dim.height);
            }
        }
    }

    /**
     * The preferred size of the image.
     */
    public Dimension getPreferredSize() 
    {
        if (null == image)
        {
            Font f = getFont();
            if (null == f) f = Font.decode("Courier-bold-30");
            FontMetrics fm = getFontMetrics(f);
            int i = Math.max(fm.getHeight()+4, fm.stringWidth((null == sIcon) ? "X" : sIcon)+4);
            return new Dimension(i, i);
        }
        else
        {
            return new Dimension(image.getWidth(this), image.getHeight(this));
        }
    }

    /**
     * The minimum size of the image.
     */  
    public Dimension getMinimumSize() 
    {
        if (null == image)
        {
            Font f = getFont();
            if (null == f) f = Font.decode("Courier-bold-30");
            FontMetrics fm = getFontMetrics(f);
            int i = Math.max(fm.getHeight()+4, fm.stringWidth((null == sIcon) ? "X" : sIcon)+4);
            return new Dimension(i, i);
        }
        else
        {
            return new Dimension(image.getWidth(this), image.getHeight(this));
        }
    }
    
    /**
     * Obtains the image specified from the resources.
     */
    private Image getImageFromResource(String sResource)
    {
        InputStream  is = getClass().getResourceAsStream(sResource);
        Image        img = null;
        MediaTracker tracker = new MediaTracker(this);

        if (null != is)
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int iRead;

            try
            {
                while(-1 != (iRead = is.read()))
                {
                    baos.write(iRead);
                }
                img = Toolkit.getDefaultToolkit().createImage(baos.toByteArray());
                tracker.addImage(img, 1);
                try
                {
                    tracker.waitForAll();
                }
                catch(InterruptedException ie)
                {
                    // System.out.println("MediaTracker interrupted");
                }
            }
            catch (IOException ioe)
            {
                img = null;
            }
        }

        return img;
    }
}
